package de.hamster.editor.controller;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import de.hamster.editor.view.EditorTools;
import de.hamster.editor.view.FileTree;
import de.hamster.editor.view.FileTreeNode;
import de.hamster.editor.view.NewHamsterDialog;
import de.hamster.editor.view.TabbedTextArea;
import de.hamster.editor.view.TextArea;
import de.hamster.model.HamsterFile;
import de.hamster.scheme.model.SchemeHamster;
import de.hamster.scheme.view.SchemeKonsole;
import de.hamster.workbench.HamsterFileFilter;
import de.hamster.workbench.Utils;
import de.hamster.workbench.Workbench;

/**
 * @author $Author: djasper $
 * @version $Revision: 1.7 $
 */
public class EditorController
		implements
			TreeSelectionListener,
			ActionListener,
			ChangeListener,
			DocumentListener {
	public static final String ACTION_NEW = "new";
	public static final String ACTION_OPEN = "open";
	public static final String ACTION_SAVE = "save";
	public static final String ACTION_SAVE_AS = "saveas";
	public static final String ACTION_CLOSE = "close";
	public static final String ACTION_PRINT = "print";
	public static final String ACTION_EXIT = "exit";
	public static final String ACTION_UNDO = "undo";
	public static final String ACTION_REDO = "redo";

	public static final String FILE_DELETE = "file_delete";
	public static final String FILE_OPEN = "file_open";
	public static final String FILE_COPY = "file_copy";
	public static final String FILE_PASTE = "file_paste";
	public static final String FILE_NEWFOLDER = "file_newfolder";
	public static final String FILE_RENAME = "file_rename";

	protected Workbench workbench;

	protected EditorTools editorTools;
	protected FileTree fileTree;
	protected TabbedTextArea tabbedTextArea;
	protected JFileChooser fileChooser;

	protected NewHamsterDialog newHamsterDialog;

	protected HamsterFile copiedFile;

	public EditorController(Workbench workbench) {
		this.workbench = workbench;

		editorTools = new EditorTools(this);
		tabbedTextArea = new TabbedTextArea(this);
		tabbedTextArea.setBackground(new Color(255, 255, 220)); // dibo 230309
		fileTree = new FileTree(this);
		fileTree.setBackground(new Color(240, 252, 202)); // dibo 230309

		newHamsterDialog = new NewHamsterDialog(tabbedTextArea);
		fileChooser = Utils.getFileChooser();
		fileChooser.setFileFilter(HamsterFileFilter.HAM_FILTER);
	}

	public FileTree getFileTree() {
		return fileTree;
	}

	public TabbedTextArea getTabbedTextArea() {
		return tabbedTextArea;
	}

	public Workbench getWorkbench() {
		return workbench;
	}

	public void markLine(HamsterFile file, int line) {
		TextArea textArea = tabbedTextArea.openFile(file);
		textArea.markLine(line);
	}

	public void markError(HamsterFile file, int line, int column) {
		TextArea textArea = tabbedTextArea.getActiveTextArea();
		textArea.requestFocus();
		textArea.selectError(line, column);
	}

	public boolean ensureSaved(HamsterFile file) {
		return tabbedTextArea.ensureSaved(file);
	}

	public void refreshFiles() {
		fileTree.update();
		tabbedTextArea.refreshFiles();
	}

	protected boolean save() {
		if (tabbedTextArea.getActiveFile().exists()) {
			tabbedTextArea.getActiveTextArea().save();
			tabbedTextArea.getActiveFile().save(tabbedTextArea.getText());
			return true;
		} else {
			return saveAs();
		}
	}

	public boolean saveAs() {
		int action = fileChooser.showSaveDialog(tabbedTextArea);
		if (action == JFileChooser.CANCEL_OPTION)
			return false;
		File f = fileChooser.getSelectedFile();
		HamsterFileFilter filter = (HamsterFileFilter) fileChooser
				.getFileFilter();
		String newName = f.getName();
		if (newName.endsWith(filter.getExtension())) {
			newName = newName.substring(0, newName.length()
					- filter.getExtension().length());
		}
		if (!Utils.isValidName(newName)) {
			Utils.message(tabbedTextArea, "editor.dialog.javanames");
			return false;
		}
		if (!f.getName().endsWith(filter.getExtension())) {
			f = new File(f.getAbsolutePath() + filter.getExtension());
		}

		HamsterFile file = tabbedTextArea.getActiveFile();
		HamsterFile newFile = HamsterFile.getHamsterFile(f.getAbsolutePath());
		newFile.setType(file.getType());
		newFile.save(tabbedTextArea.getActiveTextArea().getText());
		tabbedTextArea.closeFile(file);
		tabbedTextArea.closeFile(newFile);
		tabbedTextArea.openFile(newFile);
		return true;
	}

	protected void open() {
		int action = fileChooser.showOpenDialog(tabbedTextArea);
		if (action == JFileChooser.CANCEL_OPTION)
			return;
		File f = fileChooser.getSelectedFile();
		if (f.exists()) {
			tabbedTextArea.openFile(HamsterFile.getHamsterFile(f
					.getAbsolutePath()));
		}
	}

	protected void newFile() {
		char type = newHamsterDialog.askForType();
		if (type == (char) -1)
			return;
		// Martin
		// Falls als neue Datei die Scheme-Konsole gewhlt wird muss
		// diese angezeigt werden, und keine neue Datei.
		else if (type == HamsterFile.SCHEMEKONSOLE) {
			// SchemeKonsole ffnen
			SchemeHamster.setWorkbench(workbench);
			SchemeKonsole.getSchemeKonsole().setVisible(true);
		} else {
			HamsterFile newFile = new HamsterFile(type);
			tabbedTextArea.openFile(newFile);
			newFile.setModified(true);
		}
	}

	protected void deleteFile() {
		HamsterFile file = fileTree.getFilePopupMenu().getFile();
		if (!file.isEmpty()) {
			if (Utils.ask(tabbedTextArea, "editor.dialog.deletedir", file
					.getName())) {
				tabbedTextArea.closeFile(file);
				file.delete();
			}
		} else {
			if (Utils.ask(tabbedTextArea, "editor.dialog.reallydelete", file
					.getName())) {
				tabbedTextArea.closeFile(file);
				file.delete();
			}
		}
	}

	protected void renameFile() {
		HamsterFile oldFile = fileTree.getFilePopupMenu().getFile();

		// Namen einlesen
		String name = Utils.input(tabbedTextArea, "editor.dialog.rename");
		if (name == null || name.equals(oldFile.getName()))
			return;
		if (!oldFile.isDirectory()) {
			if (Utils.isValidName(name)) {
				name += ".ham";
			} else {
				Utils.message(tabbedTextArea, "editor.dialog.javanames");
				return;
			}
		}

		moveFile(oldFile, new File(oldFile.getFile().getParentFile(), name));
	}

	protected boolean checkOverwrite(File newFile) {
		if (newFile.exists()) {
			if (newFile.isDirectory()) {
				Utils.message(tabbedTextArea, "editor.dialog.exists", newFile
						.getName());
				return false;
			} else if (!Utils.ask(tabbedTextArea, "editor.dialog.overwrite",
					HamsterFile.getHamsterFile(newFile).getName())) {
				return false;
			} else {
				HamsterFile f = HamsterFile.getHamsterFile(newFile);
				tabbedTextArea.closeFile(f);
				f.delete();
			}
		}
		return true;
	}

	public void copyFile(HamsterFile copiedFile, HamsterFile dir) {
		File newFile = new File(dir.getFile(), copiedFile.getFile().getName());
		if (HamsterFile.getHamsterFile(newFile) != copiedFile) {
			if (tabbedTextArea.ensureSaved(copiedFile)) {
				if (checkOverwrite(newFile)) {
					copiedFile.copy(dir.getFile());
				}
			}
		}
	}

	public void moveFile(HamsterFile movedFile, File newFile) {
		if (HamsterFile.getHamsterFile(newFile) != movedFile) {
			if (tabbedTextArea.ensureSaved(movedFile)) {
				if (checkOverwrite(newFile)) {
					tabbedTextArea.closeFile(movedFile);
					movedFile.move(newFile);
				}
			}
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == ACTION_SAVE) {
			save();
		} else if (e.getActionCommand() == ACTION_NEW) {
			newFile();
		} else if (e.getActionCommand() == ACTION_SAVE_AS) {
			saveAs();
		} else if (e.getActionCommand() == ACTION_OPEN) {
			open();
		} else if (e.getActionCommand() == ACTION_CLOSE) {
			if (tabbedTextArea.ensureSaved(tabbedTextArea.getActiveFile()))
					tabbedTextArea.closeFile(tabbedTextArea.getActiveFile());
		} else if (e.getActionCommand() == ACTION_PRINT) {
			Utils.print(tabbedTextArea.getActiveTextArea().getText());
		} else if (e.getActionCommand() == ACTION_UNDO) {
			TextArea textArea = tabbedTextArea.getActiveTextArea();
			if (textArea != null) {
				if (textArea.getUndoManager().canUndo()) {
					textArea.getUndoManager().undo();
					editorTools.updateButtons(textArea);
				}
			}
		} else if (e.getActionCommand() == ACTION_REDO) {
			TextArea textArea = tabbedTextArea.getActiveTextArea();
			if (textArea != null) {
				if (textArea.getUndoManager().canRedo()) {
					textArea.getUndoManager().redo();
					editorTools.updateButtons(textArea);
				}
			}
		} else if (e.getActionCommand() == FILE_OPEN) {
			tabbedTextArea.openFile(fileTree.getFilePopupMenu().getFile());
		} else if (e.getActionCommand() == FILE_DELETE) {
			deleteFile();
		} else if (e.getActionCommand() == FILE_COPY) {
			copiedFile = fileTree.getFilePopupMenu().getFile();
		} else if (e.getActionCommand() == FILE_PASTE) {
			if (copiedFile != null && copiedFile.exists()) {
				copyFile(copiedFile, fileTree.getFilePopupMenu().getFile());
			}
		} else if (e.getActionCommand() == FILE_NEWFOLDER) {
			String name = Utils
					.input(tabbedTextArea, "editor.dialog.newfolder");
			if (name == null)
				return;
			File f = new File(fileTree.getFilePopupMenu().getFile().getFile(),
					name);
			f.mkdir();
		} else if (e.getActionCommand() == FILE_RENAME) {
			renameFile();
		}
		refreshFiles();
	}

	public void valueChanged(TreeSelectionEvent e) {
		if (e.getNewLeadSelectionPath() == null) {
			// an item was deselected
			return;
		}
		FileTreeNode f = (FileTreeNode) e.getPath().getLastPathComponent();
		HamsterFile h = f.getHamsterFile();
		if (!h.isDirectory()) {
			tabbedTextArea.openFile(h);
		}
	}

	public void stateChanged(ChangeEvent e) {
		workbench.setActiveFile(tabbedTextArea.getActiveFile());
		editorTools.setActiveFile(tabbedTextArea.getActiveFile());
		editorTools.updateButtons(tabbedTextArea.getActiveTextArea());
		if (tabbedTextArea.getActiveTextArea() != null)
			tabbedTextArea.getActiveTextArea().requestFocus();
	}

	public void changedUpdate(DocumentEvent e) {
		editorTools.updateButtons(tabbedTextArea.getActiveTextArea());
	}
	public void insertUpdate(DocumentEvent e) {
		editorTools.updateButtons(tabbedTextArea.getActiveTextArea());
	}
	public void removeUpdate(DocumentEvent e) {
		editorTools.updateButtons(tabbedTextArea.getActiveTextArea());
	}
}